﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Analysis;
using HeuristicLab.Core;
using HeuristicLab.Data;

namespace HeuristicLab.DataPreprocessing {

  public class ChartLogic : IChartLogic {
 
    private ITransactionalPreprocessingData preprocessingData;

    public ChartLogic(ITransactionalPreprocessingData preprocessingData) {
      this.preprocessingData = preprocessingData;
    }

    public DataRow CreateDataRow(string variableName, DataRowVisualProperties.DataRowChartType chartType) {
      IList<double> values = preprocessingData.GetValues<double>(variableName);
      DataRow row = new DataRow(variableName, "", values);
      row.VisualProperties.ChartType = chartType;
      return row;
    }

    public DataRow CreateDataRowRange(string variableName,int start, int end, DataRowVisualProperties.DataRowChartType chartType) {
      IList<double> values = preprocessingData.GetValues<double>(variableName);
      IList<double> valuesRange = new List<double>();
      for (int i = 0; i < values.Count; i++) {
        if (i >= start && i <= end)
          valuesRange.Add(values[i]);
        else
          valuesRange.Add(Double.NaN);
      }

      DataRow row = new DataRow(variableName, "", valuesRange);
      row.VisualProperties.ChartType = chartType;
      return row;
    }

    private void ReplaceNANwithZero(IList<double> values) {
      for (int i = 0; i < values.Count; i++) {
        if (Double.IsNaN(values[i]))
          values[i] = 0;
      }
    }



    private IEnumerable<string> GetVariableNames() {
      List<string> doubleVariableNames = new List<string>();

      //only return variable names from type double
      foreach (string variableName in preprocessingData.VariableNames) {
        if (preprocessingData.IsType<double>(preprocessingData.GetColumnIndex(variableName)))
          doubleVariableNames.Add(variableName);
      }

      return doubleVariableNames;
    }

    public ICheckedItemList<StringValue> CreateVariableItemList() {
      ICheckedItemList<StringValue> itemList = new CheckedItemList<StringValue>();
      foreach (string name in GetVariableNames()) {
        itemList.Add(new StringValue(name), true);
      }
      return new ReadOnlyCheckedItemList<StringValue>(itemList);
    }

    public event DataPreprocessingChangedEventHandler Changed {
      add { preprocessingData.Changed += value; }
      remove { preprocessingData.Changed -= value; }
    }

    public string GetVariableNameByIndex(int index) {
      return preprocessingData.GetVariableName(index);
    }


    public List<DataRow> CreateAllDataRows(DataRowVisualProperties.DataRowChartType chartType) {
      List<DataRow> dataRows = new List<DataRow>();
      foreach (var name in GetVariableNames())
        dataRows.Add(CreateDataRow(name, chartType));
      return dataRows;
    }

  }
}
